@@ -147,6 +147,9 @@ enum
 # define D_INO(dp) NOT_AN_INODE_NUMBER
 #endif
 
+/* include here for SIZE_MAX.  */
+#include <inttypes.h>
+
 /* Get or fake the disk device blocksize.
    Usually defined by sys/param.h (if at all).  */
 #if !defined DEV_BSIZE && defined BSIZE
@@ -228,8 +231,6 @@ enum
 
 #include "timespec.h"
 
-#include <inttypes.h>
-
 #include <ctype.h>
 
 #if ! (defined isblank || HAVE_DECL_ISBLANK)
@@ -627,14 +628,16 @@ emit_bug_reporting_address (void)
   fputs (_("General help using GNU software: <http://www.gnu.org/gethelp/>\n"),
 	 stdout);
 
-  /* TRANSLATORS: Replace LANG_CODE in this URL with your language code
-     <http://translationproject.org/team/LANG_CODE.html> to form one of
-     the URLs at http://translationproject.org/team/.  Otherwise, replace
-     the entire URL with your translation team's email address.  */
   if (hard_locale (LC_MESSAGES))
-    printf (_("Report %s translation bugs to "
-	      "<http://translationproject.org/team/>\n"),
-	    last_component (program_name));
+    {
+      /* TRANSLATORS: Replace LANG_CODE in this URL with your language code
+	 <http://translationproject.org/team/LANG_CODE.html> to form one of
+	 the URLs at http://translationproject.org/team/.  Otherwise, replace
+	 the entire URL with your translation team's email address.  */
+      printf (_("Report %s translation bugs to "
+		"<http://translationproject.org/team/>\n"),
+		last_component (program_name));
+    }
 }
 
 #include "inttostr.h"
@@ -653,4 +656,49 @@ bad_cast (char const *s)
   return (char *) s;
 }
 
-void usage (int status);
+/* As of Mar 2009, 32KiB is determined to be the minimium
+   blksize to best minimize system call overhead.
+   This can be tested with this script with the results
+   shown for a 1.7GHz pentium-m with 2GB of 400MHz DDR2 RAM:
+
+   for i in $(seq 0 10); do
+     size=$((8*1024**3)) #ensure this is big enough
+     bs=$((1024*2**$i))
+     printf "%7s=" $bs
+     dd bs=$bs if=/dev/zero of=/dev/null count=$(($size/$bs)) 2>&1 |
+     sed -n 's/.* \([0-9.]* [GM]B\/s\)/\1/p'
+   done
+
+      1024=734 MB/s
+      2048=1.3 GB/s
+      4096=2.4 GB/s
+      8192=3.5 GB/s
+     16384=3.9 GB/s
+     32768=5.2 GB/s
+     65536=5.3 GB/s
+    131072=5.5 GB/s
+    262144=5.7 GB/s
+    524288=5.7 GB/s
+   1048576=5.8 GB/s
+
+   Note that this is to minimize system call overhead.
+   Other values may be appropriate to minimize file system
+   or disk overhead.  For example on my current linux system
+   the readahead setting is 128KiB which was read using:
+
+   file="."
+   device=$(df -P --local "$file" | tail -n1 | cut -d' ' -f1)
+   echo $(( $(blockdev --getra $device) * 512 ))
+
+   However there isn't a portable way to get the above.
+   In the future we could use the above method if available
+   and default to io_blksize() if not.
+ */
+enum { IO_BUFSIZE = 32*1024 };
+static inline size_t
+io_blksize (struct stat sb)
+{
+  return MAX (IO_BUFSIZE, ST_BLKSIZE (sb));
+}
+
+void usage (int status) ATTRIBUTE_NORETURN;
