@@ -1,7 +1,7 @@
 /* whoami -- print effective userid
 
-   Copyright (C) 89,90, 1991-1997, 1999-2002, 2004, 2005, 2007 Free Software
-   Foundation, Inc.
+   Copyright (C) 89,90, 1991-1997, 1999-2002, 2004, 2005, 2007-2008
+   Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,10 +33,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "whoami"
 
-#define AUTHORS "Richard Mlynarik"
-
-/* The name this program was run with. */
-char *program_name;
+#define AUTHORS proper_name ("Richard Mlynarik")
 
 void
 usage (int status)
@@ -66,14 +63,14 @@ main (int argc, char **argv)
   uid_t uid;
 
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
   atexit (close_stdout);
 
-  parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE_NAME, VERSION,
+  parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE_NAME, Version,
 		      usage, AUTHORS, (char const *) NULL);
   if (getopt_long (argc, argv, "", NULL, NULL) != -1)
     usage (EXIT_FAILURE);
